package com.example.hobbid.databasedemo;

import android.content.ContentValues;
import android.content.Context;
import android.database.Cursor;
import android.database.DatabaseUtils;
import android.database.sqlite.SQLiteDatabase;
import android.database.sqlite.SQLiteOpenHelper;
import android.util.Log;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;


public class DatabaseManager extends SQLiteOpenHelper {

    private static final DatabaseStore DB_STORE = new DatabaseStore();

    /* SQLite vars */
    private static final int DATABASE_VERSION = DB_STORE.getDatabaseVersion();
    private static final String DATABASE_NAME = DB_STORE.getDatabaseName();

    /* The tables in our database */
    private static final String TABLE_NAME = DB_STORE.getTableName();


    /* Columns in the database */
    private static Map<String, String> m_columns = DB_STORE.getColumns();

    public DatabaseManager(Context context, String name, SQLiteDatabase.CursorFactory factory, int version) {
        super(context, DATABASE_NAME, factory, DATABASE_VERSION);
    }

    @Override
    public void onCreate(SQLiteDatabase db) {

        String creationQuery = "";

        creationQuery += "CREATE TABLE "     + TABLE_NAME + " (";
        creationQuery += "_id" + " INTEGER PRIMARY KEY AUTOINCREMENT";

        for(Map.Entry<String, String> entry :  m_columns.entrySet()) {

            creationQuery += ", " +  entry.getValue() + " TEXT";

        }

        creationQuery += ");";

        db.execSQL(creationQuery);

    }

    @Override
    public void onUpgrade(SQLiteDatabase db, int oldVersion, int newVersion) {

        // To upgrade the database we will simply delete the only on and create a fresh one
        db.execSQL("DROP TABLE IF EXISTS " + TABLE_NAME);
        onCreate(db);

    }

    public ArrayList<String> genLocations(){

        String queryUniqueLocations = "SELECT DISTINCT " + m_columns.get("WHERE_IS_IT_LOCATED") + " FROM " + TABLE_NAME;
        return queryDB(queryUniqueLocations);

    }

    public ArrayList<String> genRestaurants(String location){

        String queryRestaurantsInLocation = "SELECT " + m_columns.get("VENUE_NAME") + " FROM " + TABLE_NAME + " WHERE " + m_columns.get("WHERE_IS_IT_LOCATED") + " = '" + location + "'";
        return queryDB(queryRestaurantsInLocation);

    }

    public Map<String,String> genInfo(String location, String name){

        SQLiteDatabase db = getReadableDatabase();
        String s = "";

        String queryRestaurantsInfo= "SELECT * FROM " + TABLE_NAME + " WHERE " + m_columns.get("WHERE_IS_IT_LOCATED") + " = '" + location + "' AND " + m_columns.get("VENUE_NAME") + " = '" + name + "'";
        Cursor c  = db.rawQuery(queryRestaurantsInfo, null);
        c.moveToFirst();

        Map<String, String> data = new HashMap<>();

        while(!c.isAfterLast()){

            for(Map.Entry<String, String> entry :  m_columns.entrySet()) {

                Log.d("Shit",entry.getKey());
                data.put(entry.getKey(), c.getString(c.getColumnIndex(entry.getValue())));


            }

            c.moveToNext();

        }


        c.close();
        db.close();
        return data;

    }

    private ArrayList<String> queryDB(String query){

        SQLiteDatabase db = getReadableDatabase();
        ArrayList<String> al = new ArrayList< >();

        Cursor c  = db.rawQuery(query, null);
        c.moveToFirst();

        while(!c.isAfterLast()){

            al.add(c.getString(0));
            c.moveToNext();

        }


        c.close();
        db.close();
        return al;

    }

    public void populate(){

        SQLiteDatabase db = getReadableDatabase();

        if(tableIsEmpty(db)){

            /*
            *       THESE ARE THE HARDCODED ENTRIES
            *   THE DATA HERE SHOULD BE LOADED IN FROM A FILE, A URL OR COPIED ACROSS FROM A DATABASE
            *
            */

            String[] a = {"The Roundabout","Bar/ Pub","10am-Midnight","10am-Late","EASY","MAIN ROAD","WELL SIGNPOSTED","FAR","OUTDOORS AND INDOORS","NO","DRAKE CIRCUS CAR PARK","YES","YES","FRIENDLY","HELPFUL","MINIMAL CONTACT (IN A GOOD WAY)","FULLY ACCOMODATING","SIT DOWN AND ORDER AT BAR","MODERATE SELECTION","VEGETARIAN, VEGAN","DIRECTIONS (NO DIRECTIONS AS I HAVEN’T BEEN THERE)","PAY WHEN YOU ORDER","SPACIOUS","YES","FAR APART","BRIGHTLY LIT","BUSTLING ATMOSPHERE","LOUD FLAMBOYANT MUSIC","http://www.theroundaboutplymouth.co.uk/"};
            addEntry(DB_STORE.genEntry(a));

        }

    }

    private boolean tableIsEmpty(SQLiteDatabase db) {

        return DatabaseUtils.queryNumEntries(db, TABLE_NAME) == 0;

    }

    private void addEntry(Map<String,String> newVenue){

        ContentValues v = new ContentValues();

        for(Map.Entry<String, String> entry :  m_columns.entrySet()) {

            v.put(entry.getValue(), newVenue.get(entry.getKey()));

        }

        SQLiteDatabase db = getWritableDatabase();

        db.insert(TABLE_NAME, null, v);
        db.close();

    }

}
